@@ -1,6 +1,6 @@
 #!/usr/bin/python
 
-# Copyright (C) 1999 Milan Zamazal
+# Copyright (C) 1999, 2000 Milan Zamazal
 #
 # COPYRIGHT NOTICE
 #
@@ -26,7 +26,7 @@ This module is very ugly and is intended to be rewritten sometimes.
 """
 
 __author__  = "Milan Zamazal <pdm@freesoft.cz>"
-__version__ = "$Id: cgiscript.py,v 1.20 2000/01/09 19:38:27 pdm Exp $"
+__version__ = "$Id: cgiscript.py,v 1.24 2000/01/25 20:40:59 pdm Exp $"
 __copyright__ = "GNU General Public License, version 2"
 
 
@@ -48,7 +48,7 @@ from gnats2w.i18n import L
 # Constants
 
 
-VERSION = '0.12'
+VERSION = '0.14'
 
 
 # Reporting errors
@@ -151,6 +151,7 @@ def html_dump (problem):
     """
     # Generate dictionary
     items = problem.items ()
+    void = '---'
     for i, j in (('submitter', 'submitter-id'),
                  ('arrived', 'arrival-date'),
                  ('closed', 'closed-date'),
@@ -169,7 +170,7 @@ def html_dump (problem):
               'priority', 'arrived', 'modified', 'closed', 'description',
               'repeat', 'fix', 'environment', 'note', 'trail', 'unformatted'):
         if not items.has_key (k) or not items[k]:
-            items[k] = '---'
+            items[k] = void
     for k in ('description', 'repeat', 'fix', 'environment', 'note', 'trail',
               'unformatted'):
         items[k] = HTMLgen.Pre (items[k])
@@ -178,6 +179,8 @@ def html_dump (problem):
     subject = urllib.quote (items['synopsis'])
     _, remail = problem.responsible ()
     if remail:
+        if items['responsible'] == void:
+            items['responsible'] = remail
         items['responsible'] = HTMLgen.Href ('mailto:%s?subject=%s' % \
                                              (remail, subject),
                                              items['responsible'])
@@ -186,6 +189,8 @@ def html_dump (problem):
     except KeyError:
         oemail = ''
     if oemail:
+        if items['originator'] == void:
+            items['originator'] = oemail
         items['originator'] = HTMLgen.Href ('mailto:%s?subject=%s' % \
                                             (oemail, subject),
                                             items['originator'])
@@ -258,9 +263,8 @@ def make_edit_dict (problem, id):
         cats.append (cat)
     cats.sort ()
     all = (cats,
-           ['open', 'analyzed', 'feedback', 'closed', 'suspended'],
-           ['sw-bug', 'doc-bug', 'change-request', 'support', 'duplicate',
-            'mistaken'],
+           G.states (),
+           G.classes (),
            ['critical', 'serious', 'non-critical'],
            ['high', 'medium', 'low'])
     for i in range (len (all)):
@@ -287,7 +291,9 @@ def make_edit_dict (problem, id):
             'priority': str (list[4]),
             'config': config,
             'dump': dump,
-            'catcgi': '%s' % gen_submit_url ('categories')
+            'catcgi': '%s' % gen_submit_url ('categories'),
+            'clacgi': '%s' % gen_submit_url ('classes'),
+            'stacgi': '%s' % gen_submit_url ('states')
             }
 
 
@@ -418,12 +424,16 @@ def submit (form):
         categories.sort ()
         html_categories = HTMLgen.Select (categories, name='category',
                                           selected=[config.DEFAULT_CATEGORY])
+        html_classes = HTMLgen.Select (G.classes (), name='class')
         output_document ('submit',
                          {'categories': str (html_categories),
+                          'classes': str (html_classes),
                           'confidentialp': config.DISPLAY_CONFIDENTIAL,
                           'submitter': config.ACCESS.name (),
                           'submittere': config.ACCESS.email (),
-                          'catcgi': '%s' % gen_submit_url ('categories')})
+                          'catcgi': '%s' % gen_submit_url ('categories'),
+                          'clacgi': '%s' % gen_submit_url ('classes'),
+                          'stacgi': '%s' % gen_submit_url ('states')})
         return
     # Filled report
     form = form2dict (form)
@@ -509,7 +519,7 @@ def edit (form):
             if k == 'category' and \
                (not form.has_key ('responsible')
                 or form['responsible'] == items['responsible']):
-                cats = G.categories_full ()
+                cats = categories_full ()
                 for c in cats:
                     if c[0] == v:
                         r = c[3]
@@ -666,13 +676,21 @@ def query (form):
         categories.sort ()
         categories = HTMLgen.Select (categories, name='category', size=6,
                                      multiple=1)
+        classes = HTMLgen.Select (G.classes (), name='class', size=6,
+                                  multiple=1)
         responsible = HTMLgen.Select (G.responsible (), name='responsible',
                                       size=6, multiple=1)
+        states = HTMLgen.Select (G.states (), name='state', size=5,
+                                 multiple=1)
         output_document ('query',
                          {'categories': str (categories),
+                          'classes': str (classes),
                           'responsible': str (responsible),
+                          'states': str (states),
                           'confidentialp': config.DISPLAY_CONFIDENTIAL,
-                          'catcgi': '%s' % gen_submit_url ('categories')})
+                          'catcgi': '%s' % gen_submit_url ('categories'),
+                          'clacgi': '%s' % gen_submit_url ('classes'),
+                          'stacgi': '%s' % gen_submit_url ('states')})
         return
     # Real query
     items = map (lambda m: (m.name, m.value), form.list)
@@ -716,8 +734,7 @@ def query (form):
                        funcid (string.atoi (x[0]), string.atoi (y[0]))
         elif sortby == 'state':
             def func (x, y):
-                return funcspec (['open', 'analyzed', 'feedback', 'closed',
-                                  'suspended'],
+                return funcspec (G.states (),
                                  x[3], y[3]) or \
                        funcspec (['critical', 'serious', 'non-critical'],
                                  x[4], y[4]) or \
@@ -866,7 +883,10 @@ def categories (form):
         row.append (HTMLgen.TD (L(c[2])))
         name, email = G.id2email (c[3])
         if not name:
-            name = '?'
+            if email:
+                name = email
+            else:
+                name = '?'
         if email:
             who = HTMLgen.Href ('mailto:%s' % email, name)
         else:
@@ -874,7 +894,55 @@ def categories (form):
         row.append (HTMLgen.TD (who))
     # Output it
     output_document ('categories', {'categories': str (table)})
+
+
+def classes (form):
+    """Create list of classes with description.
+    """
+    # Access permitted?
+    access = config.ACCESS.granted (config.ACCESS.SUBMIT) \
+             or config.ACCESS.granted (config.ACCESS.VIEW)
+    if not access:
+        report_access_denied (config.ACCESS.VIEW)
+        return
+    # Create classes list
+    table = HTMLgen.TableLite ()
+    row = HTMLgen.TR ()
+    table.append (row)
+    for i in (L('Class'), L('Description')):
+        row.append (HTMLgen.TD (HTMLgen.Strong (i)))
+    for c in G.classes_full ():
+        row = HTMLgen.TR ()
+        table.append (row)
+        row.append (HTMLgen.TD (c[0]))
+        row.append (HTMLgen.TD (L(c[2])))
+    # Output it
+    output_document ('classes', {'classes': str (table)})
+
     
+def states (form):
+    """Create list of states with description.
+    """
+    # Access permitted?
+    access = config.ACCESS.granted (config.ACCESS.SUBMIT) \
+             or config.ACCESS.granted (config.ACCESS.VIEW)
+    if not access:
+        report_access_denied (config.ACCESS.VIEW)
+        return
+    # Create states list
+    table = HTMLgen.TableLite ()
+    row = HTMLgen.TR ()
+    table.append (row)
+    for i in (L('State'), L('Description')):
+        row.append (HTMLgen.TD (HTMLgen.Strong (i)))
+    for c in G.states_full ():
+        row = HTMLgen.TR ()
+        table.append (row)
+        row.append (HTMLgen.TD (c[0]))
+        row.append (HTMLgen.TD (L(c[2])))
+    # Output it
+    output_document ('states', {'states': str (table)})
+
 
 # Top level function
 
@@ -926,6 +994,10 @@ def go (form=None):
             request (form)
         elif script_name == 'categories':
             categories (form)
+        elif script_name == 'classes':
+            classes (form)
+        elif script_name == 'states':
+            states (form)
         else:
             report_error (L('Unknown request: %s') % script_name)
     except:
