@@ -27,7 +27,7 @@ GNATS databases are accessed only by GNATS commands (both public and internal).
 """
 
 __author__  = "Milan Zamazal <pdm@freesoft.cz>"
-__version__ = "$Id: gnats.py,v 1.15 2000/01/12 19:36:14 pdm Exp $"
+__version__ = "$Id: gnats.py,v 1.17 2000/01/22 20:22:26 pdm Exp $"
 __copyright__ = "GNU General Public License, version 2"
 
 
@@ -128,6 +128,31 @@ class Gnats:
             str = string.replace (str, dangerous, ".")
         return str
 
+
+    def _extract_first (self, list, sort=0):
+        """Return list of the first elements of 'list' sublists.
+
+        If 'list' is 'None', return 'None'.
+        Iff sort is true, sort the resulting list.
+        """
+        if list == None:
+            return None
+        items = map (lambda i: i[0], list)
+        if sort:
+            items.sort ()
+        return items
+
+
+    def _directory_command (self, command):
+        """Add appropriate '--directory' to 'command' if needed.
+
+        Return the resulting string.
+        """
+        if self._directory:
+            command = command + (" --directory='%s' " % \
+                                 self._protect (self._directory))
+        return command
+        
         
     def _get_problem (self, id):
         """Get raw text of the problem # 'id'.
@@ -135,11 +160,7 @@ class Gnats:
         Return string.
         If the problem cannot be found or accessed, return 'None'.
         """
-        if self._directory:
-            command = "query-pr --directory='%s'" % \
-                      self._protect (self._directory)
-        else:
-            command = "query-pr"
+        command = self._directory_command ('query-pr')
         try:
             f = os.popen (command + ' --full %s' % self._protect (str (id)))
             text = f.read ()
@@ -202,9 +223,7 @@ class Gnats:
         """
         # Compose locking command
         command = os.path.join (config.GNATS_PRG_DIR, 'pr-edit') + ' '
-        if self._directory:
-            command = command + "--directory='%s' " % \
-                      self._protect (self._directory)
+        command = self._directory_command (command)
         if id:
             command = command + "--lock=www '%s'" % self._protect (str (id))
         else:
@@ -242,9 +261,7 @@ class Gnats:
         """
         # Compose unlocking command
         command = os.path.join (config.GNATS_PRG_DIR, 'pr-edit') + ' '
-        if self._directory:
-            command = command + "--directory='%s' " % \
-                      self._protect (self._directory)
+        command = self._directory_command (command)
         if id:
             command = command + "--unlock '%s'" % self._protect (str (id))
         else:
@@ -337,10 +354,7 @@ class Gnats:
 
         If the list cannot be retrieved, return 'None'.
         """
-        command = 'query-pr --list-categories'
-        if self._directory:
-            command = "%s --directory='%s'" % \
-                      (command, self._protect (self._site))
+        command = self._directory_command ('query-pr --list-categories')
         result = []
         try:
             f = os.popen (command)
@@ -364,19 +378,54 @@ class Gnats:
         except:
             return None
         return result
-        
+
+
+    def classes (self):
+        """Return list of all classes for the GNATS db.
+
+        If it cannot be retrieved, return 'None'.
+        """
+        return self._extract_first (self.classes_full ())
+
+
+    def classes_full (self):
+        """Return a list of all information about responsible persons.
+
+        The list contains lists of the form [CLASS, TYPE, DESCRIPTION].
+        CLASS is the class name.
+        TYPE is type of the class, by default 'class'.
+        DESCRIPTION is description of the class.
+
+        If the list cannot be retrieved, return 'None'.
+        """
+        command = self._directory_command ('query-pr --list-classes')
+        try:
+            items = []
+            f = os.popen (command)
+            while 1:
+                line = f.readline ()
+                if not line:
+                    break
+                if line[0] == '#':
+                    continue
+                line = string.rstrip (line)
+                line = string.split (line, ':')
+                if len (line) >= 3:
+                    if not line[1]:
+                        line[1] = 'class'
+                    items.append (line)
+            f.close ()
+            return items
+        except:
+            return None        
+
             
     def responsible (self):
         """Return a sorted list of all responsible persons for the GNATS db.
 
         If it cannot be retrieved, return 'None'.
         """
-        items = self.responsible_full ()
-        if items == None:
-            return None
-        items = map (lambda i: i[0], items)
-        items.sort ()
-        return items
+        return self._extract_first (self.responsible_full (), 1)
     
 
     def responsible_full (self):
@@ -389,10 +438,7 @@ class Gnats:
 
         If the list cannot be retrieved, return 'None'.
         """
-        command = 'query-pr --list-responsible'
-        if self._directory:
-            command = "%s --directory='%s'" % \
-                      (command, self._protect (self._directory))
+        command = self._directory_command ('query-pr --list-responsible')
         try:
             items = []
             f = os.popen (command)
@@ -415,15 +461,48 @@ class Gnats:
             return None
 
 
+    def states (self):
+        """Return list of all states of GNATS db.
+        """
+        return self._extract_first (self.states_full ())
+
+
+    def states_full (self):
+        """Return a list of all information about responsible persons.
+
+        The list contains lists of the form [STATE, TYPE, DESCRIPTION].
+        STATE is the name of the state.
+        TYPE is type of the state, by default `open'.
+        DESCRIPTION is description of the state.
+
+        If the list cannot be retrieved, return 'None'.
+        """
+        command = self._directory_command ('query-pr --list-states')
+        try:
+            items = []
+            f = os.popen (command)
+            while 1:
+                line = f.readline ()
+                if not line:
+                    break
+                if line[0] == '#':
+                    continue
+                line = string.rstrip (line)
+                line = string.split (line, ':')
+                if len (line) >= 3:
+                    if not line[1]:
+                        line[1] = 'open'
+                    items.append (line)
+            f.close ()
+            return items
+        except:
+            return None
+
+    
     def submitters (self):
         """Return sorted list of submitters.
         """
-        submitters = self.submitters_full ()
-        if not submitters:
-            return None
-        submitters = map (lambda s: s[0], submitters)
-        submitters.sort ()
-        return submitters
+        return self._extract_first (self.submitters_full (), 1)
     
 
     def submitters_full (self):
@@ -442,10 +521,7 @@ class Gnats:
 
         If the information about submitters cannot be retrieved, return 'None'.
         """
-        command = 'query-pr --list-submitters'
-        if self._directory:
-            command = "%s --directory='%s'" % \
-                      (command, self._protect (self._directory))
+        command = self._directory_command ('query-pr --list-submitters')
         try:
             items = []
             f = os.popen (command)
@@ -527,9 +603,7 @@ class Gnats:
         """
         id = problem.id ()
         command = os.path.join (config.GNATS_PRG_DIR, 'pr-edit')
-        if self._directory:
-            command = command + " --directory='%s'" % \
-                      self._protect (self._directory)
+        command = self._directory_command (command)
         command = "%s '%s'" % (command, self._protect (id))
         # Lock problem
         result = self._lock (id)
@@ -581,8 +655,10 @@ class Gnats:
 
         Return whether it succeeded.
         """
-        f = os.popen (os.path.join (config.GNATS_PRG_DIR, 'queue-pr') + ' -q',
-                      'w')
+        command = os.path.join (config.GNATS_PRG_DIR, 'queue-pr')
+        command = self._directory_command (command)
+        command = command + ' -q'
+        f = os.popen (command, 'w')
         f.write (self.dump (problem))
         result = f.close ()
         return not result
@@ -641,11 +717,7 @@ class Gnats:
         if query == '':
             query = "--text='.'"
         # Perform query
-        if self._directory:
-            command = "query-pr --summary --directory='%s' " % \
-                      self._protect (self._directory)
-        else:
-            command = "query-pr --summary "
+        command = self._directory_command ('query-pr --summary ')
         try:
             f, dummy, err = popen2.popen3 (command + query)
             dummy.close ()
